package com.mortals.xhx.module.device.service;

import com.mortals.framework.common.Rest;
import com.mortals.framework.model.Context;
import com.mortals.framework.service.ICRUDCacheService;
import com.mortals.xhx.busiz.req.DeviceMsgReq;
import com.mortals.xhx.busiz.rsp.ApiResp;
import com.mortals.xhx.busiz.rsp.ServerInfo;
import com.mortals.xhx.common.code.DeviceMethodEnum;
import com.mortals.xhx.module.device.dao.DeviceDao;
import com.mortals.xhx.module.device.model.DeviceEntity;
import com.mortals.xhx.module.device.model.DeviceMapEntity;
import com.mortals.xhx.module.platform.model.PlatformEntity;
import com.mortals.xhx.module.product.model.ProductEntity;
import com.mortals.xhx.queue.TbQueueCallback;
import com.mortals.xhx.queue.TbQueueMsgHeaders;
import com.mortals.xhx.queue.TopicPartitionInfo;

import java.util.List;

/**
* DeviceService
*
* 设备 service接口
*
* @author zxfei
* @date 2022-03-09
*/
public interface DeviceService extends ICRUDCacheService<DeviceEntity,Long>{

    DeviceDao getDeviceDao();

    /**
     *
     * @param deviceEntity
     * @param info
     * @param header
     * @param message
     * @param callback
     * @param context
     * @return
     */
    ApiResp<String> sendDeviceMessage(DeviceEntity deviceEntity, TopicPartitionInfo info, TbQueueMsgHeaders header, String message, TbQueueCallback callback , Context context);

    /**
     *
     * @param deviceIds
     * @param info
     * @param header
     * @param message
     * @param context
     * @return
     */
    ApiResp<String> sendDeviceMessage(List<Long> deviceIds, TopicPartitionInfo info,TbQueueMsgHeaders header, String message , Context context);

    /**
     * 设备激活
     * @param deviceCode
     * @param context
     */
    void active(String deviceCode,Context context);

    /**
     * 设备重启应用
     * @param deviceEntity
     * @param context
     */
    void restartApp(DeviceEntity deviceEntity,Context context);

    /**
     *
     * @param deviceEntity
     * @param context
     */
    void batchUpdateHall(DeviceEntity deviceEntity,Context context);

    /**
     * 设备启用停用
     * @param id
     * @param context
     */
    void deviceEnabled(Long id,Integer status,Context context);

    /**
     *
     * @param id
     * @param status
     * @param context
     */
    void stopOrStartComsumeQueue(Long id,Integer status,Context context);

    /**
     *
     * @param entity
     * @param productEntity
     * @param platformEntity
     * @param update
     * @return
     */
    Rest<String> sendThirdParty(DeviceEntity entity, ProductEntity productEntity, PlatformEntity platformEntity, DeviceMethodEnum update);

    /**
     *
     * @param key
     * @param deviceEntity
     */
    void putCache(String key,DeviceEntity deviceEntity);

    /**
     *
     * @param query
     * @param context
     * @return
     */
    List<DeviceMapEntity> deviceMap(DeviceEntity query, Context context);

    /**
     *
     * @param list
     */
    void downMsg(List<DeviceMsgReq> list);

    /**
     *
     * @param deviceEntity
     * @param productEntity
     * @param serverInfo
     * @param serverName
     * @param serverPort
     */
    void buildHomeUrlBySite(DeviceEntity deviceEntity, ProductEntity productEntity, ServerInfo serverInfo, String serverName, Integer serverPort);


}